﻿using System;
using System.Collections.Generic;

namespace Enumerable_examples {
   public static class Compute {
      public static IEnumerable<int> Range(int Start, int End, int Step) {
         if (Step > 0)
            for (int i = Start; i <= End; i += Step)
               yield return i;
         else if (Step < 0)
            for (int i = Start; i >= End; i += Step)
               yield return i;
         yield break;
      }

      public static IEnumerable<int> Range(int Start, int End) {
         if (Start <= End)
            return Range(Start, End, 1);
         else
            return Range(Start, End, -1);
      }

      public static IEnumerable<double> Range(double Start, double End, double Step) {
         if (Step > 0)
            for (double i = Start; i <= End; i += Step)
               yield return i;
         else if (Step < 0)
            for (double i = Start; i >= End; i += Step)
               yield return i;
         yield break;
      }

      public static IEnumerable<double> Range(double Start, double End) {
         if (Start <= End)
            return Range(Start, End, 1);
         else
            return Range(Start, End, -1);
      }

      public static IEnumerable<int> Counter(int Start, int Step) {
         for (int i = Start; ; Start += Step)
            yield return i;
      }

      public static IEnumerable<int> Counter(int Start) {
         return Counter(Start, 1);
      }

      public static IEnumerable<int> Random(int Min, int Max, int Seed) {
         Random rand = new Random(Seed);
         for (; ; )
            yield return rand.Next(Min, Max);
      }

      public static IEnumerable<int> Random(int Min, int Max) {
         Random rand = new Random();
         for (; ; )
            yield return rand.Next(Min, Max);
      }

      public static IEnumerable<Guid> RandomGuid() {
         for (; ; )
            yield return Guid.NewGuid();
      }

      public static int Count(IEnumerable List) {
         int Count = 0;
         foreach (object o in List)
            Count++;
         return Count;
      }

      public static int Sum(IEnumerable<int> List) {
         int Sum = 0;
         foreach (int x in List)
            Sum += x;
         return Sum;
      }

      public static double Sum(IEnumerable<double> List) {
         double Sum = 0;
         foreach (double x in List)
            Sum += x;
         return Sum;
      }
      public static int Average(IEnumerable<int> List) {
         int Counter = 0;
         int Sum = 0;
         foreach (int x in List) {
            Counter++;
            Sum += x;
         }
         return Sum / Counter;
      }

      public static double Average(IEnumerable<double> List) {
         double Counter = 0;
         double Sum = 0;
         foreach (double x in List) {
            Counter++;
            Sum += x;
         }
         return Sum / Counter;
      }

      public static int Minimum(IEnumerable<int> List) {
         int Min = int.MaxValue;
         foreach (int x in List)
            if (x < Min) Min = x;
         return Min;
      }

      public static double Minimum(IEnumerable<double> List) {
         double Min = double.MaxValue;
         foreach (double x in List)
            if (x < Min) Min = x;
         return Min;
      }

      public static int Maximum(IEnumerable<int> List) {
         int Max = int.MinValue;
         foreach (int x in List)
            if (x > Max) Max = x;
         return Max;
      }

      public static double Maximum(IEnumerable<double> List) {
         double Max = double.MinValue;
         foreach (double x in List)
            if (x > Max) Max = x;
         return Max;
      }
   }
}
