﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;

namespace Enumerable_examples {
   public static class Filter {
      public static IEnumerable<T> Grep<T>(IEnumerable<T> List, Predicate<T> FilterDelegate) {
         foreach (T o in List)
            if (FilterDelegate(o)) yield return o;
         yield break;
      }

      public static IEnumerable<T> Splice<T>(IEnumerable<T> List, int OffsetIndex, int Count) {
         return Skip<T>(Take<T>(List, Count), OffsetIndex);
         //int Offset = 0;
         //int ReturnCount = 0;

         //foreach (T o in List) {
         //   if (Offset < OffsetIndex) {
         //      Offset++;
         //   }
         //   else {
         //      yield return o;
         //      ReturnCount++;
         //   }
         //   if (ReturnCount >= Count) yield break;
         //}
      }

      public static IEnumerable<T> Splice<T>(IEnumerable<T> List, int Count) {
         return Splice<T>(List, 0, Count);
      }

      public static IEnumerable<T> Skip<T>(IEnumerable<T> List, int Count) {
         int Offset = 0;

         foreach (T o in List)
            if (Offset < Count)
               Offset++;
            else
               yield return o;

         yield break;
      }

      public static IEnumerable<T> Take<T>(IEnumerable<T> List, int Count) {
         int Offset = -1;
         foreach (T o in List) {
            Offset++;
            if (Offset >= Count) break;
            yield return o;
         }
         yield break;
      }

      public static IEnumerable<T> Tail<T>(IEnumerable<T> List, int Count) {
         Queue<T> QueueList = new Queue<T>(Count);

         foreach (T o in List) {
            if (QueueList.Count == Count) QueueList.Dequeue();
            QueueList.Enqueue(o);
         }

         foreach (T o in QueueList)
            yield return o;

         yield break;
      }

      public enum CompareOperator { LessThan = -1, Equals = 0, GreaterThan = -1 }

      public static IEnumerable<T> CompareTo<T>(IEnumerable<T> List, CompareOperator Operator, T Subject) where T : IComparable<T> {
         foreach (T o in List)
            if (((IComparable<T>)o).CompareTo(Subject) == (int)Operator) yield return o;
      }

      public static IEnumerable<T> CompareToOrEquals<T>(IEnumerable<T> List, CompareOperator Operator, T Subject) where T : IComparable<T> {
         foreach (T o in List) {
            int CompareResult = ((IComparable<T>)o).CompareTo(Subject);
            if (CompareResult == 0 || CompareResult == (int)Operator) yield return o;
         }
      }
   }
}
